package com.xdocletbook.portlets;

import java.io.IOException;
import java.io.PrintWriter;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletContext;
import javax.portlet.PortletException;
import javax.portlet.PortletMode;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequestDispatcher;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.portlet.ValidatorException;

/**
 * @portlet.portlet
 *     name="WeatherPortlet"
 *     description="Weather Forecast Portlet"
 *     display-name="Weather Forecast"
 *     expiration-cache="3600"
 * 
 * @portlet.supports
 *     mime-type="text/html"
 *     modes="EDIT"
 * 
 * @portlet.portlet-init-param
 *     name="BaseUrl"
 *     value="http://banners.wunderground.com/banner/infobox/US/"
 * 
 * @portlet.preference
 *     name="city"
 *     value="Dallas"
 * 
 * @portlet.preference
 *     name="state"
 *     value="TX"
 * 
 * @portlet.preference-validator
 *     class="com.xdocletbook.portlets.WeatherPreferenceValidator"
 * 
 * @portlet.portlet-info
 *     title="Weather Portlet"
 */
public class WeatherPortlet extends GenericPortlet {

  public void processAction(ActionRequest request,
      ActionResponse response) 
      throws PortletException, IOException {

    if(request.getPortletMode().equals(PortletMode.EDIT)) {
      String city = request.getParameter("city");
      String state = request.getParameter("state");
      
      PortletPreferences prefs = request.getPreferences();
      prefs.setValue("city", city);
      prefs.setValue("state", state);
      
      try {
        prefs.store();
        response.setPortletMode(PortletMode.VIEW);
      } catch (ValidatorException e) {
        response.setRenderParameter("error", e.getMessage());
      }
    }
  }


  public void doView(RenderRequest request, RenderResponse response)
      throws PortletException, IOException {

    PortletPreferences prefs = request.getPreferences();
    String city = prefs.getValue("city", "Dallas");
    String state = prefs.getValue("state", "TX");

    String weatherUrl = getInitParameter("BaseUrl") + 
        state + "/" + city + ".gif";        
        
    response.setContentType("text/html");
    
    PrintWriter writer = response.getWriter();
    writer.write("<img src='"+weatherUrl+"'>");
  }
  
  public void doEdit(RenderRequest request, RenderResponse response)
      throws PortletException, IOException {

    PortletPreferences prefs = request.getPreferences();
    request.setAttribute("city", prefs.getValue("city", "Dallas"));
    request.setAttribute("state", prefs.getValue("state", "TX"));
    
    PortletContext context = getPortletContext();
    PortletRequestDispatcher dispatch = 
        context.getRequestDispatcher(
        "/WEB-INF/jsp/html/WeatherEdit.jsp");
        
    dispatch.include(request, response);
  }
}
