package com.xdocletbook.blog.ejb;

import java.util.Set;

import javax.ejb.CreateException;
import javax.ejb.EntityBean;

import com.xdocletbook.blog.interfaces.BlogLocal;
import com.xdocletbook.blog.util.TopicUtil;
import com.xdocletbook.blog.value.TopicValue;

/**
 * An entity bean representing a blog topic.
 * 
 * @ejb.bean
 *     name="Topic"
 *     type="CMP"
 *     cmp-version="2.x"
 *     primkey-field="id"
 *     view-type="local"
 * 
 * @ejb.value-object
 * 
 * @ejb.finder
 *    signature="java.util.Collection findAll()"
 *    query="SELECT OBJECT(t) FROM Topic AS t"
 */
public abstract class TopicBean implements EntityBean {
  
  //
  // CREATE METHODS
  //
  /** @ejb.create-method */
  public String ejbCreate(String name, BlogLocal blog) throws CreateException {
        
    this.setId(TopicUtil.generateGUID(this));
    this.setName(name);

    return null;
  }
  
  public void ejbPostCreate(String name, BlogLocal blog) throws CreateException {
    // Set CMR fields
    this.setBlog(blog);
  }

  /** @ejb.create-method */
  public String ejbCreate(TopicValue topicValue) throws CreateException {
        
    this.setId(TopicUtil.generateGUID(this));
    this.setName(topicValue.getName());
    
    return null;
  }
  
  public void ejbPostCreate(TopicValue topicValue) throws CreateException {
  }

  //
  // CMP FIELDS
  //
  /**
   * ID (Primary Key)
   * 
   * @ejb.persistence
   * @ejb.interface-method
   */
  public abstract String getId();
  public abstract void setId(String id);

  /**
   * Name
   * 
   * @ejb.persistence
   * @ejb.interface-method
   */ 
  public abstract String getName();
  /** @ejb.interface-method */
  public abstract void setName(String name);
  
  
  //
  // CMR FIELDS
  //
  /**
   * @ejb.interface-method
   *
   * @ejb.relation
   *     name="topic-entries"
   *     role-name="topic-has-entries"
   * 
   * @ejb.value-object
   *     aggregate="com.xdocletbook.blog.value.EntryValue"
   *     aggregate-name="Entry"
   *     members="com.xdocletbook.blog.interfaces.EntryLocal"
   *     members-name="Entries"
   *     relation="external"
   *     type="java.util.Set"
   */
  public abstract Set getEntries();
  
  /** @ejb.interface-method */
  public abstract void setEntries(Set entries);  
  
  /**
   * @ejb.interface-method
   * 
   * @ejb.relation
   *     name="blog-topics"
   *     role-name="topic-belongs_to-blog"
   *     cascade-delete="yes"
   */
  public abstract BlogLocal getBlog();
  
  /** @ejb.interface-method */
  public abstract void setBlog(BlogLocal blog);


  //
  // BUSINESS METHODS
  //
  /** @ejb.interface-method */
  public abstract TopicValue getTopicValue();
  
  /** @ejb.interface-method */
  public void setTopicValue(TopicValue topicValue) {
    this.setName(topicValue.getName());
  }
}
