package com.xdocletbook.blog.servlet;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.xdocletbook.blog.exception.ApplicationException;
import com.xdocletbook.blog.exception.InvalidUserException;
import com.xdocletbook.blog.interfaces.BlogFacade;
import com.xdocletbook.blog.interfaces.BlogFacadeHome;
import com.xdocletbook.blog.util.BlogFacadeUtil;
import com.xdocletbook.blog.value.BlogSimpleValue;

/**
 * @web.servlet
 *     name="CreateTopic"
 *     run-as="blogowner"
 *
 * @web.servlet-mapping
 *     url-pattern="/owner/createTopic"
 * 
 * @web.ejb-ref 
 *     name="ejb/BlogFacade"
 *     type="session"
 *     home="com.xdoclet.blog.interfaces.BlogFacadeHome"
 *     remote="com.xdoclet.blog.interfaces.BlogFacade"
 *     link="BlogFacade"
 * 
 * @web.security-role
 *     role-name="${OwnerRole}"
 * 
 * @web.security-role-ref
 *     role-name="blogowner"
 *     role-link="${OwnerRole}"
 */
public class CreateTopicServlet extends HttpServlet {
  public void service(HttpServletRequest request, 
      HttpServletResponse response)
      throws ServletException, IOException {

    String blogId = request.getParameter("blogId");
    String name = request.getParameter("name");
    
    try {
      BlogFacadeHome home = BlogFacadeUtil.getHome();
      BlogFacade blogFacade = home.create();
      BlogSimpleValue blog = blogFacade.getBlogSimple(blogId);
      String userName = request.getUserPrincipal().getName();
      String blogOwner = blog.getOwner();
         
      if(request.isUserInRole("blogowner") && 
          userName.equals(blogOwner)) {
        blogFacade.createTopic(name, blogId);
             
        request.getRequestDispatcher(
            "/jsp/topicCreated.jsp").forward(request, response);
      } else {
        request.setAttribute("userException", 
            new InvalidUserException(userName, blogOwner));

        request.getRequestDispatcher(
            "/jsp/badUser.jsp").forward(request, response);
      }
    } catch (Exception e) {
      e.printStackTrace();
      request.setAttribute("appException", 
          new ApplicationException(e));
      request.getRequestDispatcher(
          "/jsp/error.jsp").forward(request, response);
    }
  }
}
