package com.xdocletbook.blog.servlet;

import java.io.IOException;
import java.util.Collections;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.xdocletbook.blog.exception.ApplicationException;
import com.xdocletbook.blog.interfaces.BlogFacade;
import com.xdocletbook.blog.interfaces.BlogFacadeHome;
import com.xdocletbook.blog.util.BlogFacadeUtil;
import com.xdocletbook.blog.util.EntryValueComparator;

/**
 * @web.servlet
 *     name="DisplayEntries"
 *
 * @web.servlet-mapping
 *     url-pattern="/listEntries"
 * 
 * @web.ejb-ref 
 *     name="ejb/BlogFacade"
 *     type="session"
 *     home="com.xdoclet.blog.interfaces.BlogFacadeHome"
 *     remote="com.xdoclet.blog.interfaces.BlogFacade"
 *     link="BlogFacade"
 */
public class ListEntriesServlet extends HttpServlet {
  public void service(HttpServletRequest request, 
      HttpServletResponse response)
      throws ServletException, IOException {

    try {
      String blogId = request.getParameter("blogId");
  
      BlogFacadeHome home = BlogFacadeUtil.getHome();
      BlogFacade facade = home.create();
      
      request.setAttribute("blog", facade.getBlogSimple(blogId));
      
      List entries = facade.getRecentEntries(blogId, 10);
      Collections.sort(entries, new EntryValueComparator());
      request.setAttribute("entries", entries);
          
      request.getRequestDispatcher(
          "jsp/entryList.jsp").forward(request, response);
    } catch (Exception e) {
      e.printStackTrace();
      
      request.setAttribute("appException", 
          new ApplicationException(e));
      request.getRequestDispatcher(
          "jsp/error.jsp").forward(request, response);
    }
  }
}
