package com.xdocletbook.blog.ejb;

import java.util.Set;

import javax.ejb.CreateException;
import javax.ejb.EntityBean;

import com.xdocletbook.blog.util.BlogUtil;
import com.xdocletbook.blog.value.BlogSimpleValue;
import com.xdocletbook.blog.value.BlogValue;

/**
 * An entity bean representing a blog (weblog).
 * 
 * @ejb.bean
 *     name="Blog"
 *     type="CMP"
 *     cmp-version="2.x"
 *     primkey-field="id"
 *     view-type="local"
 * 
 * @ejb.value-object
 * 
 * @ejb.value-object
 *     name="BlogSimple"
 *     match="simple"
 * 
 * @ejb.finder
 *     signature="java.util.Collection findAll()"
 *     query="SELECT OBJECT(b) FROM Blog AS b"
 * 
 * @struts.form
 *     name="Everything"
 *     include-all="true"
 */
public abstract class BlogBean implements EntityBean {
  //
  // CREATE METHODS
  //
  /** @ejb.create-method */
  public String ejbCreate(String name, String owner, String email) 
      throws CreateException {
        
    this.setId(BlogUtil.generateGUID(this));
    this.setName(name);
    this.setOwner(owner);
    this.setEmail(email);
    
    return null;
  }
  
  public void ejbPostCreate(String name, String owner, String email) 
      throws CreateException {
  }
  
  //
  // CMP FIELDS
  //
  /**
   * ID (Primary Key)
   * 
   * @ejb.persistence
   * @ejb.interface-method
   * @ejb.value-object
   *     match="simple"
   */
  public abstract String getId();
  public abstract void setId(String id);
  
  /**
   * Name of the blog.
   * 
   * @ejb.persistence
   * @ejb.interface-method
   * @ejb.value-object
   *     match="simple"
   * 
   * @struts.form-field form-name="Create"
   *
   * @struts.validator
   *     type="buster"
   *     msgkey="max"
   *     arg0value="buster"
   *     arg0resource="false"
   */
  public abstract String getName();
  
  /** @ejb.interface-method */
  public abstract void setName(String name);
  
  /**
   * Owner of the blog.
   * 
   * @ejb.persistence
   * @ejb.interface-method
   * @ejb.value-object
   *     match="simple"
   * 
   * @struts.form-field form-name="Create"
   */
  public abstract String getOwner();
  
  /** @ejb.interface-method */
  public abstract void setOwner(String owner);

  /**
   * Contact e-mail
   * 
   * @ejb.persistence
   * @ejb.interface-method
   * 
   * @struts.form-field form-name="Create"
   */
  public abstract String getEmail();
  
  /** @ejb.interface-method */
  public abstract void setEmail(String email);
  
  //
  // CMR FIELDS
  //
  /**
   * @ejb.interface-method
   *
   * @ejb.relation
   *     name="blog-entries"
   *     role-name="blog-has-entries"
   * 
   * @ejb.value-object
   *     aggregate="com.xdocletbook.blog.value.EntryValue"
   *     aggregate-name="Entry"
   *     members="com.xdocletbook.blog.interfaces.EntryLocal"
   *     members-name="Entries"
   *     relation="external"
   *     type="java.util.Set"
   */
  public abstract Set getEntries();
  
  /** @ejb.interface-method */
  public abstract void setEntries(Set entries);
  
  /**
   * @ejb.interface-method
   * 
   * @ejb.relation
   *     name="blog-topics"
   *     role-name="blog-has-topics"
   * 
   * @ejb.value-object
   *     aggregate="com.xdocletbook.blog.value.TopicValue"
   *     aggregate-name="Topic"
   *     members="com.xdocletbook.blog.interfaces.TopicLocal"
   *     members-name="Topics"
   *     relation="external"
   *     type="java.util.Set"
   */
  public abstract Set getTopics();
  
  /** @ejb.interface-method */
  public abstract void setTopics(Set topics);
  
  //
  // BUSINESS METHODS
  //
  /** @ejb.interface-method */
  public abstract BlogValue getBlogValue();
  
  /** @ejb.interface-method */
  public abstract BlogSimpleValue getBlogSimpleValue();
}
