package com.xdocletbook.blog.filter;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import com.xdocletbook.blog.interfaces.BlogFacade;
import com.xdocletbook.blog.util.BlogFacadeUtil;
import com.xdocletbook.blog.util.BlogSimpleValueComparator;

/**
 * @web.filter
 *     name="BlogLoader"
 * 
 * @web.filter-mapping
 *     url-pattern="/*"
 * 
 * @web.ejb-ref 
 *     name="ejb/BlogFacade"
 *     type="session"
 *     home="com.xdoclet.blog.interfaces.BlogFacadeHome"
 *     remote="com.xdoclet.blog.interfaces.BlogFacade"
 *     link="BlogFacade"
 */
public class BlogLoadFilter implements Filter{
  private static Logger LOGGER = 
      Logger.getLogger(BlogLoadFilter.class);

  public void init(ServletConfig config) throws ServletException {
    String logLevel = config.getInitParameter("LogLevel");
    if(logLevel.equals("debug")) {
      LOGGER.setLevel(Level.DEBUG);
    } else if(logLevel.equals("info")) {
      LOGGER.setLevel(Level.INFO);
    } else if(logLevel.equals("warn")) {
      LOGGER.setLevel(Level.WARN);
    } else {
      // Leave log level at default
    }
  }

  FilterConfig filterConfig;
  
  public void init(FilterConfig filterConfig)
      throws ServletException {
    this.filterConfig = filterConfig;
  }

  public void doFilter(ServletRequest request, 
      ServletResponse response, FilterChain chain) 
      throws IOException, ServletException {

    try {
      BlogFacade blogFacade = BlogFacadeUtil.getHome().create();
     
      List blogs = blogFacade.getAllBlogs();
      List allBlogs = new ArrayList(blogs.size());
      for(Iterator iter = blogs.iterator(); iter.hasNext(); ) {
        allBlogs.add(iter.next());
      }

      Collections.sort(allBlogs, new BlogSimpleValueComparator());

      request.setAttribute("allBlogs", allBlogs);
    } catch (Exception e) {
      request.getRequestDispatcher("/jsp/error.jsp").
          forward(request, response);
    }

    chain.doFilter(request, response);
  }

  public void destroy() {
  }
}
