package com.xdocletbook.blog.pojo;

import java.util.Set;

/** 
 * @hibernate.class
 * 
 * @jdo.persistence-capable
 *     identity-type="application"
 *     objectid-class="java.lang.String"
 * 
 * @sql.table table-name="Blog"
 * 
 * @castor.class
 *   id="id"
 *   table="Blog"
 *   key-generator="UUID"
 */
public class Blog {
  /**
   * @jdo.field
   *     primary-key="true"
   * 
   * @tjdo.field column-length="32" 
   * 
   * @kodo.field column-length="64"
   * 
   * @sql.field column-name="blog_id"
   */
  private String id;
  
  /**
   * @jdo.field
   *     null-value="exception"
   * 
   * @tjdo.field column-length="20" 
   */
  private String name;
  
  /**
   * @jdo.field
   *     null-value="exception"
   * 
   * 
   * @tjdo.field column-length="30" 
   */
  private String owner;
  
  /**
   * @jdo.field
   *     null-value="exception"
   * 
   * @tjdo.field column-length="20" 
   */
  private String email;
  
  /**
   * {@jdo.field
   *     collection-type="collection"
   *     element-type="com.xdocletbook.blog.pojo.Topic"}
   */
  private Set topics;
  
  /**
   * {@jdo.field
   *     collection-type="collection"
   *     element-type="com.xdocletbook.blog.pojo.Entry"}
   */
  private Set entries;
   
  public Blog() {
  }

  /**
   * @hibernate.id 
   *     generator-class="uuid.string"
   * @castor.field
   *     set-method="setId"
   * 
   * @castor.field-sql
   *     name="id"
   *     type="char"
   * 
   * @castor.field-xml
   *     node="attribute"
   */
  public String getId() {
    return id;
  }
  
  /**
   * @hibernate.property
   * 
   * @castor.field
   *     set-method="setEmail"
   * 
   * @castor.field-sql
   *     name="email"
   *     type="char"
   * 
   * @castor.field-xml
   *     name="contact"
   */
  public String getEmail() {
    return email;
  }

  /**
   * @hibernate.property 
   *   not-null="true"
   * 
   * @castor.field
   *     set-method="setName"
   * 
   * @castor.field-sql
   *     name="name"
   *     type="char"
   */
  public String getName() {
    return name;
  }

  /**
   * @hibernate.property
   *
   * @castor.field
   *     set-method="setOwner"
   * 
   * @castor.field-sql
   *     name="owner"
   *     type="char"
   * 
   * @castor.field-xml
   *     name="belongsTo"
   */
  public String getOwner() {
    return owner;
  }

  public void setEmail(String string) {
    email = string;
  }

  public void setId(String string) {
    id = string;
  }

  public void setName(String string) {
    name = string;
  }

  public void setOwner(String string) {
    owner = string;
  }
  
  /**
   * @hibernate.set
   *     lazy="true"
   *     cascade="all"
   * 
   * @hibernate.collection-key
   *     column="blog"
   * 
   * @hibernate.collection-one-to-many
   *     class="com.xdocletbook.blog.pojo.Entry"
   * 
   * @castor.field
   *     collection="set"
   *     type="com.xdocletbook.blog.pojo.Entry"
   *     set-method="setEntries"
   * 
   * @castor.field-sql
   *     many-key="blog"
   * 
   * @castor.field-xml
   *     name="entry"
   */
  public Set getEntries() {
    return entries;
  }

  public void setEntries(Set set) {
    entries = set;
  }

  /**
   * @hibernate.set
   *     lazy="true"
   *     cascade="all"
   * 
   * @hibernate.collection-key
   *     column="blog"
   *     generator-class="uuid.string"
   * 
   * @hibernate.collection-one-to-many
   *     class="com.xdocletbook.blog.pojo.Topic"
   * 
   * @castor.field
   *     collection="set"
   *     type="com.xdocletbook.blog.pojo.Topic"
   *     set-method="setTopics"
   * 
   * @castor.field-sql
   *     many-key="blog"
   * 
   * @castor.field-xml
   *     name="topic"
   */
  public Set getTopics() {
    return topics;
  }

  public void setTopics(Set set) {
    topics = set;
  }
}
