package com.xdocletbook.blog.pojo;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.exolab.castor.jdo.ClassNotPersistenceCapableException;
import org.exolab.castor.jdo.Database;
import org.exolab.castor.jdo.JDO;
import org.exolab.castor.jdo.LockNotGrantedException;
import org.exolab.castor.jdo.OQLQuery;
import org.exolab.castor.jdo.ObjectNotFoundException;
import org.exolab.castor.jdo.PersistenceException;
import org.exolab.castor.jdo.QueryResults;
import org.exolab.castor.jdo.TransactionNotInProgressException;

import com.xdocletbook.blog.exception.ApplicationException;

public class BlogFacadeCastor implements BlogFacade {
  private static final String DB_NAME = "blog";
  private JDO jdo;
  private Database db;
  
  public static BlogFacadeCastor getInstance(String configFile) 
      throws ApplicationException {
    return new BlogFacadeCastor(configFile);
  }
  
  private BlogFacadeCastor(String configFile) 
      throws ApplicationException {
    jdo = new JDO();
    jdo.setConfiguration(configFile);
    jdo.setClassLoader(getClass().getClassLoader());
    jdo.setDatabaseName(DB_NAME);
    
    try {
      db = jdo.getDatabase();
    } catch (PersistenceException e) {
      throw new ApplicationException(e);
    }
  }
 
  // -------------------- BLOG METHODS ------------------
  public Blog createBlog(String name, String owner, String email) 
      throws ApplicationException {
         
    Blog blog = new Blog();
    blog.setName(name);
    blog.setOwner(owner);
    blog.setEmail(email);
     
    try {
      db.begin();
      db.create(blog);
      db.commit();
    } catch(PersistenceException e) {
      throw new ApplicationException(e);
    } finally {
      rollback(db);
    }
     
    return blog;
  }
  
  public Blog getBlog(String blogId) throws ApplicationException {
    try {
      db.begin();
      return (Blog)db.load(Blog.class, blogId);
    } catch (PersistenceException e) {
      throw new ApplicationException(e);
    } finally {
      rollback(db);
    }
  }
  
  public Collection getAllBlogs() throws ApplicationException {
    String select = 
        "SELECT b FROM com.xdocletbook.blog.pojo.Blog b";
    
    try {
      db.begin();
      
      OQLQuery query = db.getOQLQuery(select);
      QueryResults results = query.execute();
      
      List resultsList = new ArrayList();
      while(results.hasMore()) {
        resultsList.add(results.next());
      }

      return resultsList;
    } catch (PersistenceException e) {
      throw new ApplicationException(e);
    } finally {
      rollback(db);
    }
  }
  
  public void updateBlog(Blog blog) throws ApplicationException {
    try {
      db.begin();
      db.update(blog);  // XXX - this probably won't work because
                        // XXX - this is a different transaction
      db.commit();
    } catch (PersistenceException e) {
      rollback(db);
      throw new ApplicationException(e);
    }
  }
   
  public void deleteBlog(String blogId) 
      throws ApplicationException {
         
    try {
      db.begin();
      Blog blog = (Blog) db.load(Blog.class, blogId);
      db.remove(blog);
      db.commit();
    } catch (PersistenceException e) {
      rollback(db);
      throw new ApplicationException(e);
    }
  }
   
   public Collection getTopics(String blogId)
       throws ApplicationException {
   
     try {
       db.begin();
       Blog blog = (Blog) db.load(Blog.class, blogId);  
       return blog.getTopics();
     } catch (PersistenceException e) {
       throw new ApplicationException(e);
     } finally {
       rollback(db);
     }
   }
   
   private void rollback(Database db) {
     try {
       db.rollback();
     } catch (Exception e) {}
   }
   
   protected void finalize() throws Throwable {
    db.close();
  }

}
