package com.xdocletbook.blog.pojo;

import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.Properties;
import javax.jdo.JDOFatalInternalException;
import javax.jdo.JDOHelper;
import javax.jdo.PersistenceManager;
import javax.jdo.PersistenceManagerFactory;
import javax.jdo.Transaction;

import com.xdocletbook.blog.exception.ApplicationException;

public class BlogFacadeJdo implements BlogFacade {

  private static PersistenceManagerFactory factory = null;
  private PersistenceManager pm = null; 

  public static PersistenceManagerFactory 
      getPersistenceManagerFactory () {
    if (factory == null) {
      Properties props = new Properties ();
      try {
        InputStream in =
            ClassLoader.getSystemResourceAsStream("jdo.properties");
        props.load (in);
      } catch (IOException ioe) {
        throw new JDOFatalInternalException(
            "Cannot load property file'", ioe);
      }
      factory = JDOHelper.getPersistenceManagerFactory (props);
    }
    return factory;
  }
  
  private PersistenceManager getPersistenceManager () {
   if (pm == null) {
     pm =  getPersistenceManagerFactory ().getPersistenceManager();
   }
   return pm;
  }
  
  public Blog createBlog(String name, String owner, String email)
      throws ApplicationException {
       
    Transaction tx = pm.currentTransaction();
    try { 
      Blog blog = new Blog();
      blog.setName(name);
      blog.setOwner(owner);
      blog.setEmail(email);
      
      getPersistenceManager().makePersistent(blog);
      tx.commit();
      
      return blog;
    } finally {
      if(tx.isActive()) {
        tx.rollback();
      }
    }
  }
  
  public void deleteBlog(String blogId)
      throws ApplicationException {
  
    Transaction tx = pm.currentTransaction();
    try { 
      Blog blog = (Blog)pm.getObjectById(blogId, false);
      pm.deletePersistent(blog);
      tx.commit();
    } finally {
      if(tx.isActive()) {
        tx.rollback();
      }
    }
  }

  public Collection getAllBlogs() throws ApplicationException {
    return (Collection)getPersistenceManager().newQuery().execute();
  }

  public Collection getTopics(String blogId)
      throws ApplicationException {

    Blog blog = 
        (Blog)getPersistenceManager().getObjectById(blogId, false);

    return blog.getTopics();
  }
}
