package com.xdocletbook.blog.pojo;

import java.io.Serializable;
import java.sql.SQLException;
import java.util.List;

import net.sf.hibernate.HibernateException;
import net.sf.hibernate.ObjectNotFoundException;
import net.sf.hibernate.Session;
import net.sf.hibernate.SessionFactory;
import net.sf.hibernate.cfg.Configuration;
import net.sf.hibernate.type.Type;

public class HibernateUtils {
  private static Configuration configuration;
  private static SessionFactory sessionFactory;

  private static final ThreadLocal threadLocal = new ThreadLocal();
  
  static {
    try {
      configuration = new Configuration();
      
//      configuration.setProperty("hibernate.connection.driver_class", "org.gjt.mm.mysql.Driver");
//      configuration.setProperty("hibernate.connection.url", "jdbc:mysql://127.0.0.1/jboss");
//      configuration.setProperty("hibernate.dialect", "net.sf.hibernate.dialect.MySQLDialect");
//      configuration.setProperty("hibernate.default_schema", "jboss");
      
    configuration.addFile("d:/projects/xdocletbook/generated/hbm/com/xdocletbook/blog/pojo/Blog.hbm.xml");
    configuration.addFile("d:/projects/xdocletbook/generated/hbm/com/xdocletbook/blog/pojo/Entry.hbm.xml");
    configuration.addFile("d:/projects/xdocletbook/generated/hbm/com/xdocletbook/blog/pojo/Topic.hbm.xml");
    
//      configuration.addResource(
//          "d:/projects/xdocletbook/generated/hbm/com/xdocletbook/blog/pojo/Blog.hbm.xml", 
//          HibernateUtils.class.getClassLoader());
//      configuration.addResource(
//          "d:/projects/xdocletbook/generated/hbm/com/xdocletbook/blog/pojo/Entry.hbm.xml", 
//          HibernateUtils.class.getClassLoader());
//      configuration.addResource(
//          "d:/projects/xdocletbook/generated/hbm/com/xdocletbook/blog/pojo/Topic.hbm.xml", 
//          HibernateUtils.class.getClassLoader());
      sessionFactory = configuration.buildSessionFactory();
    } catch (Exception e) {
      System.err.println(e);
      e.printStackTrace();
      throw new RuntimeException(e.getMessage());
    }
  }

  public static void createThreadLocalSession() 
      throws HibernateException {
    threadLocal.set(getSession());
  }
  
  
  public static void finalizeThreadLocalSession() 
      throws HibernateException, SQLException {

    Session threadLocalSession = (Session) threadLocal.get();
    flush(threadLocalSession);
    commit(threadLocalSession);
    close(threadLocalSession);

    threadLocal.set(null);
  }  

  public static Object load(Class clazz, Serializable id) 
      throws HibernateException {
        
    Session session = getSession();
        
    try {

      try {
        return getSession().load(clazz, id);
      } catch (ObjectNotFoundException e) {
        return null;
      }
    } 
    finally {
      closeNonThreadLocalSession(session);
    }
  }
  
  
  public static List find(String query) throws HibernateException {

    Session session = getSession();

    try {
      return session.find(query);
    } finally {
      closeNonThreadLocalSession(session);
    }
  }
  
  
  public static Object findByAlternateKey(String query) 
      throws HibernateException {

    List objectList = find(query);
    
    if (objectList == null || objectList.size() == 0) {
      return null;
    }
    return objectList.get(0);
  }
  
  
  public static List find(String query, Object value, Type type) 
      throws HibernateException {
    
    Session session = getSession();
    
    try {
      return session.find(query, value, type);
    } finally {
      closeNonThreadLocalSession(session);
    }
  }
  
  public static Object findByAlternateKey(String query, 
      Object value, Type type) throws HibernateException {

    List objectList = find(query, value, type);
    
    if (objectList == null || objectList.size() == 0) {
      return null;
    }
    return objectList.get(0);
  }

  public static Object findByAlternateKey(String query, 
      Object[] values, Type[] types) throws HibernateException {

    List objectList = find(query, values, types);
    
    if (objectList == null || objectList.size() == 0) {
      return null;
    }
    return objectList.get(0);
  }
  
  
  public static List find(String query, Object[] values, 
      Type[] types) throws HibernateException {
    
    Session session = getSession();

    try {
      return session.find(query, values, types);
    } 
    finally {
      closeNonThreadLocalSession(session);
    }
  }

  public static void save(Object object) throws HibernateException {
    Session session = getSession();
        
    try {
      session.save(object);
      flushNonThreadLocalSession(session);
      commitNonThreadLocalSession(session);
    } catch (HibernateException e) {
      rollback(session);
      throw e;
    } finally {
      closeNonThreadLocalSession(session);
    }
  }
  
  
  public static void update(Object object) 
      throws HibernateException {
    
    Session session = null;
        
    try {
      session = getSession();
      session.update(object);
      flushNonThreadLocalSession(session);
      commitNonThreadLocalSession(session);
    } catch (HibernateException e) {
      rollback(session);
      throw e;
    } finally {
      closeNonThreadLocalSession(session);
    }
  }
  
  
  public static void delete(Object object)
      throws HibernateException {
    
    Session session = null;
        
    try {
      session = getSession();
      session.delete(object);
      flushNonThreadLocalSession(session);
      commitNonThreadLocalSession(session);
    } catch (HibernateException e) {
      rollback(session);
      throw e;
    } 
    finally {
      closeNonThreadLocalSession(session);
    }
  }

  private static Session getSession() throws HibernateException {
    if (threadLocal.get() == null) {
      return sessionFactory.openSession();
    } else {
      return (Session) threadLocal.get();
    }
  }
  
  
  private static void closeNonThreadLocalSession(Session session) {
    if (threadLocal.get() != null && threadLocal.get() == session) {
      return;
    }
    
    close(session);
  }
  
  
  private static void close(Session session) {
    
    if (session == null || session.isOpen() == false) {
      return;
    }
    
    try {
      session.close();
    } catch (HibernateException e) {
    }
  }
  
  
  private static void flushNonThreadLocalSession(Session session) {
    if (threadLocal.get() != null && threadLocal.get() == session) {
      return;
    }
    
    close(session);
  }
  
  
  private static void flush(Session session) 
      throws HibernateException {
    
    if (session == null || session.isOpen() == false) {
      return;
    }

    session.flush();
  }
  
  
  private static void commitNonThreadLocalSession(Session session) {
    if (threadLocal.get() != null && threadLocal.get() == session) {
      return;
    }
    
    close(session);
  }
  
  
  private static void commit(Session session) 
      throws HibernateException, SQLException {
    
    if (session == null || session.isOpen() == false) {
      return;
    }
    
    session.connection().commit();
  }
  
  
  private static void rollback(Session session) {
    
    if (session == null) {
      return;
    }
    
    try {
      session.connection().rollback();
    } catch (HibernateException e) {
    } catch (SQLException e) {
    }
  }
  
  
  private static boolean threadLocalSessionExists() {
    return (threadLocal.get() == null);
  }

}