package com.xdocletbook.jmx;

import java.util.List;

import com.xdocletbook.blog.interfaces.BlogFacade;
import com.xdocletbook.blog.interfaces.BlogFacadeHome;
import com.xdocletbook.blog.util.BlogFacadeUtil;

/**
 * @jmx.mbean
 *    name="BlogManager"
 *    description="Some basic administrative functions for blogs."
 * 
 * @jboss.mxbean
 */
public class BlogManager implements BlogManagerMBean {
  BlogFacadeHome home;

  public BlogManager() throws Exception {
    home = BlogFacadeUtil.getHome();
  }

  /**
   * @jmx.managed-operation
   *     description="Retrieve a blog's name given its ID."
   * 
   * @jmx.managed-operation-parameter
   *     description="The ID of the blog to be looked up."
   *     name="blogId"
   *     position="1"
   */
  public String getBlogName(String blogId) throws Exception {
    BlogFacade facade = home.create();
    return facade.getBlogSimple(blogId).getName();
  }
  
  /**
   * @jmx.managed-operation
   *     description="Delete a blog."
   * 
   * @jmx.managed-operation-parameter
   *     description="The ID of the blog to be deleted."
   *     name="blogId"
   *     position="1"
   */
  public void deleteBlog(String blogId) throws Exception {
    home.create().deleteBlog(blogId);
  }
  
  /**
   * @jmx.managed-operation
   *     description="Retrieve a list of all blogs."
   */
  public List getAllBlogs() throws Exception{
    BlogFacade facade = home.create();
    return facade.getAllBlogs();
  }
}
